/*
 * libspe - A wrapper library to adapt the JSRE SPU usage model to SPUFS
 * Copyright (C) 2005 IBM Corp.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 *  License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software Foundation,
 *   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define _GNU_SOURCE
#include <errno.h>
#include <libspe.h>
#include <pthread.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <sys/mman.h>

#include "spe.h"

#define __PRINTF(fmt, args...) { fprintf(stderr,fmt , ## args); }
#ifdef DEBUG
#define DEBUG_PRINTF(fmt, args...) __PRINTF(fmt , ## args)
#else
#define DEBUG_PRINTF(fmt, args...)
#endif

/* Internal function prototypes. only used internaly.
 */
void *map_mfc_area(speid_t speid);
void *map_cntl_area(speid_t speid);
void *map_signal1_area(speid_t speid);
void *map_signal2_area(speid_t speid);

void *map_mfc_area(speid_t speid)
{
	struct thread_store *spe = speid;

	if (spe->mfc_mmap_base == MAP_FAILED)
	{
		spe->mfc_mmap_base = mmap(0, 0x4000, PROT_READ | PROT_WRITE, MAP_SHARED, spe->fd_mfc, 0);
	}

	return spe->mfc_mmap_base;
}

void *map_cntl_area(speid_t speid)
{
	struct thread_store *spe = speid;

	if (spe->cntl_mmap_base == MAP_FAILED)
	{
		spe->cntl_mmap_base = mmap(0, 0x1000, PROT_READ | PROT_WRITE, MAP_SHARED, spe->fd_cntl, 0);
	}

	return spe->cntl_mmap_base;
}

void *map_signal1_area(speid_t speid)
{
	struct thread_store *spe = speid;

	if (spe->signal1_mmap_base == MAP_FAILED)
	{
		spe->signal1_mmap_base = mmap(0, 0x1000, PROT_READ | PROT_WRITE, MAP_SHARED, spe->fd_sig1, 0);
	}

	return spe->signal1_mmap_base;
}

void *map_signal2_area(speid_t speid)
{
	struct thread_store *spe = speid;

	if (spe->signal2_mmap_base == MAP_FAILED)
	{
		spe->signal2_mmap_base = mmap(0, 0x1000, PROT_READ | PROT_WRITE, MAP_SHARED, spe->fd_sig2, 0);
	}

	return spe->signal2_mmap_base;
}

void *spe_get_ps_area(speid_t speid, enum ps_area area)
{
	void *base_addr;

	if ( area == SPE_MSSYNC_AREA )
	{
		base_addr = map_mfc_area(speid);

		if (base_addr == MAP_FAILED)
		{
                        perror("spe_map_ps_area: internal error. cant map mfc file.");
                        errno = EIO;
			return MAP_FAILED;
		}
		else
		{
			return base_addr; 
		}
	}
	else if ( area == SPE_MFC_COMMAND_AREA )
	{
		base_addr = map_mfc_area(speid);

		if (base_addr == MAP_FAILED)
		{
                        perror("spe_map_ps_area: internal error. cant map mfc file.");
                        errno = EIO;
			return MAP_FAILED;
		}
		else
		{
			return base_addr + 0x3000; 
		}

	}
	else if ( area == SPE_CONTROL_AREA )
	{
		base_addr = map_cntl_area(speid);

		if (base_addr == MAP_FAILED)
		{
                        perror("spe_map_ps_area: internal error. cant map control file.");
                        errno = EIO;
			return MAP_FAILED;
		}
		else
		{
			return base_addr ; 
		}
	}
	else if ( area == SPE_SIG_NOTIFY_1_AREA )
	{
		base_addr = map_signal1_area(speid);

		if (base_addr == MAP_FAILED)
		{
                        perror("spe_map_ps_area: internal error. cant map signal1 file.");
                        errno = EIO;
			return MAP_FAILED;
		}
		else
		{
			return base_addr; 
		}
	}
	else if ( area == SPE_SIG_NOTIFY_2_AREA )
	{
		base_addr = map_signal2_area(speid);

		if (base_addr == MAP_FAILED)
		{
                        perror("spe_map_ps_area: internal error. cant map signal2 file.");
                        errno = EIO;
			return MAP_FAILED;
		}
		else
		{
			return base_addr; 
		}
	}
	
	perror("spe_map_ps_area: Unsupported call of spe_map_ps_area.");
	errno = EIO;
	return MAP_FAILED;
	
}
